﻿using System;
using System.Diagnostics;
using System.Globalization;
using System.Windows.Forms;
using ThinkGeo.MapSuite.Core;
using ThinkGeo.MapSuite.DesktopEdition;
using ThinkGeo.MapSuite.Routing;
using System.Collections.ObjectModel;

namespace TravelingSalesmanProblem
{
    public partial class Form1 : Form
    {
        private static RoutingEngine routingEngine;
        private static RoutingSource routingSource;
        private static TspRoutingResult routingResult;

        public Form1()
        {
            InitializeComponent();
        }

        private void button1_Click(object sender, EventArgs e)
        {
            LayerOverlay overlay = (LayerOverlay)winformsMap1.Overlays["RoutingOverlay"];
            RoutingLayer routingLayer = (RoutingLayer)overlay.Layers["RoutingLayer"];

            Stopwatch watch = new Stopwatch();
            watch.Start();
            //New API for GetRouteViaVisitstops wher you can have a start and an end point.
            routingResult = routingEngine.GetRouteViaVisitStops(routingLayer.StartPoint, routingLayer.EndPoint, routingLayer.StopPoints, int.Parse(txtIterations.Text));

            watch.Stop();
            // Render the route
            routingLayer.Routes.Clear();
            routingLayer.Routes.Add(routingResult.Route);
            winformsMap1.Refresh(winformsMap1.Overlays["RoutingOverlay"]);

            routingLayer.StopPoints.Clear();
            foreach (PointShape stop in routingResult.VisitSequence)
            {
                routingLayer.StopPoints.Add(stop);
            }
            routingLayer.ShowStopOrder = true;
            txtTimeUsed.Text = watch.ElapsedMilliseconds.ToString(CultureInfo.InvariantCulture) + " ms";
            txtDistance.Text = routingResult.Distance.ToString(CultureInfo.InvariantCulture);
            winformsMap1.Refresh(overlay);
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            ShapeFileFeatureSource featureSource = new ShapeFileFeatureSource(@"..\..\data\Austinstreets.shp");
            routingSource = new RtgRoutingSource(@"..\..\data\Austinstreets.rtg");
            routingEngine = new RoutingEngine(routingSource, new AStarRoutingAlgorithm(), featureSource);

            RenderMap();
        }

        private void RenderMap()
        {
            winformsMap1.MapUnit = GeographyUnit.DecimalDegree;
            winformsMap1.BackgroundOverlay.BackgroundBrush = new GeoSolidBrush(GeoColor.FromHtml("#e6e5d1"));
            winformsMap1.CurrentExtent = new RectangleShape(-97.7319606824951, 30.2922109418945, -97.7002033277588, 30.269551640136);

            WorldMapKitWmsDesktopOverlay worldMapKitsOverlay = new WorldMapKitWmsDesktopOverlay();
            winformsMap1.Overlays.Add(worldMapKitsOverlay);

            RoutingLayer routingLayer = new RoutingLayer();
            LayerOverlay routingOverlay = new LayerOverlay();
            winformsMap1.Overlays.Add("RoutingOverlay", routingOverlay);
            routingOverlay.Layers.Add("RoutingLayer", routingLayer);

            string[] startCoordinates = txtStart.Text.Split(',');
            routingLayer.StartPoint = new PointShape(double.Parse(startCoordinates[0], CultureInfo.InvariantCulture), double.Parse(startCoordinates[1], CultureInfo.InvariantCulture));
            string[] endCoordinates = EndTxt.Text.Split(',');
            routingLayer.EndPoint = new PointShape(double.Parse(endCoordinates[0], CultureInfo.InvariantCulture), double.Parse(endCoordinates[1], CultureInfo.InvariantCulture));

            foreach (object item in lsbPoints.Items)
            {
                string[] coordinate = item.ToString().Split(',');
                PointShape pointNeedVisit = new PointShape(double.Parse(coordinate[0], CultureInfo.InvariantCulture), double.Parse(coordinate[1], CultureInfo.InvariantCulture));
                routingLayer.StopPoints.Add(pointNeedVisit);
            }
            routingLayer.ShowStopOrder = false;

            InMemoryFeatureLayer routingExtentLayer = new InMemoryFeatureLayer();
            routingExtentLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle = new AreaStyle(new GeoPen(GeoColor.SimpleColors.Green));
            routingExtentLayer.ZoomLevelSet.ZoomLevel01.DefaultLineStyle = LineStyles.Railway1;
            routingExtentLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;
            routingExtentLayer.InternalFeatures.Add(new Feature(new RectangleShape(-97.815409, 30.369949, -97.657999, 30.217922)));
            routingOverlay.Layers.Add("RoutingExtentLayer", routingExtentLayer);

            winformsMap1.Refresh();
        }

        private void button2_Click(object sender, EventArgs e)
        {
            LayerOverlay overlay = (LayerOverlay)winformsMap1.Overlays["RoutingOverlay"];
            RoutingLayer routingLayer = (RoutingLayer)overlay.Layers["RoutingLayer"];

            int stopCount = routingLayer.StopPoints.Count;
            Random random = new Random(DateTime.Now.Millisecond);
            int randomStopIndex = random.Next(1, stopCount - 1);
            label7.Text = randomStopIndex.ToString();
            
            PointShape startPoint = routingResult.VisitSequence[randomStopIndex];
            PointShape endPoint = routingResult.VisitSequence[randomStopIndex + 1];

            RectangleShape startBox = GenerateMiniBoundingBox(startPoint);
            RectangleShape endBox = GenerateMiniBoundingBox(endPoint);

            Collection<RouteSegment> pairSegments = new Collection<RouteSegment>();
            Collection<LineShape> lines = new Collection<LineShape>();
            bool isMiddleSegment = false;
            for (int i = 0; i < routingResult.Route.Lines.Count; i++)
            {
                LineShape line = routingResult.Route.Lines[i];
                if (startBox.Intersects(line))
                {
                    pairSegments.Add(routingResult.RouteSegments[i]);
                    isMiddleSegment = true;
                }
                if (endBox.Intersects(line))
                {
                    pairSegments.Add(routingResult.RouteSegments[i]);
                    lines.Add(line);
                    isMiddleSegment = false;
                    break;
                }
                if (isMiddleSegment)
                {
                    lines.Add(line);
                }
            }

            InMemoryFeatureLayer layer = overlay.Layers["RoutingExtentLayer"] as InMemoryFeatureLayer;
            layer.Open();
            layer.InternalFeatures.Clear();
            foreach (LineShape item in lines)
            {
                layer.InternalFeatures.Add(new Feature(item));
            };

            winformsMap1.Refresh(overlay);
        }

        private RectangleShape GenerateMiniBoundingBox(PointShape point)
        {
            RectangleShape bbox = new RectangleShape();
            double miniTolerance = 1e-3/ 2;
            bbox.UpperLeftPoint.X = point.X - miniTolerance;
            bbox.UpperLeftPoint.Y = point.Y + miniTolerance;
            bbox.LowerRightPoint.X = point.X + miniTolerance;
            bbox.LowerRightPoint.Y = point.Y - miniTolerance;

            return bbox;
        }
    }
}
